unit exever;

interface

uses Classes, SysUtils, Windows, Registry;

type
  TExeVersion = record
    case Integer of
      0: (verMS, verLS: DWORD);
      1: (verW2, verW1, verW4, verW3: WORD);
  end;

function GetExeVersion(AModule: HMODULE = 0): TExeVersion;
function GetExeVersionDescription(AModule: HMODULE = 0): string;

implementation

function GetExeVersion(AModule: HMODULE): TExeVersion;
var
  rsrc, glob: THandle;
  ptr: Pointer;
  ffi: PVSFIXEDFILEINFO;
  len: UINT;
  mem: TMemoryStream;
begin
  try
    rsrc := FindResource(AModule, MakeIntResource(1), RT_VERSION);
    if rsrc <> 0 then begin
      glob := LoadResource(AModule, rsrc);
      if glob <> 0 then try
        ptr := LockResource(glob);
        if ptr <> nil then begin
          mem := TMemoryStream.Create;
          try
            mem.Position := 0;
            mem.WriteBuffer(ptr^, SizeofResource(AModule, rsrc));
            if VerQueryValue(mem.Memory, '\', Pointer(ffi), len) then begin
              Result.verMS := ffi.dwFileVersionMS;
              Result.verLS := ffi.dwFileVersionLS;
            end;
          finally
            mem.Free;
          end;
        end;
      finally
        FreeResource(glob);
      end;
    end;
  except
    //
  end;
end;

{------------------------------------------------------------------------------}

function GetExeVersionDescription(AModule: HMODULE): string;
var
  rsrc, glob: THandle;
  ptr: Pointer;
  ver: TExeVersion;
  ffi: PVSFIXEDFILEINFO;
  len: UINT;
  attr: string;
  mem: TMemoryStream;
begin
  try
    rsrc := FindResource(AModule, MakeIntResource(1), RT_VERSION);
    if rsrc <> 0 then begin
      glob := LoadResource(AModule, rsrc);
      if glob <> 0 then try
        ptr := LockResource(glob);
        if ptr <> nil then begin
          mem := TMemoryStream.Create;
          try
            mem.Position := 0;
            mem.WriteBuffer(ptr^, SizeofResource(AModule, rsrc));
            if VerQueryValue(mem.Memory, '\', Pointer(ffi), len) then begin
              ver.verMS := ffi.dwFileVersionMS;
              ver.verLS := ffi.dwFileVersionLS;
              Result := Format('%d.%d.%d.%d', [ver.verW1, ver.verW2, ver.verW3, ver.verW4]);
              if (ffi^.dwFileFlagsMask and VS_FF_DEBUG <> 0) and
              (ffi^.dwFileFlags and VS_FF_DEBUG <> 0) then
                attr := ',debug';
              if (ffi^.dwFileFlagsMask and VS_FF_PRERELEASE <> 0) and
              (ffi^.dwFileFlags and VS_FF_PRERELEASE <> 0) then
                attr := attr + ',prerelease';
              if (ffi^.dwFileFlagsMask and VS_FF_PATCHED <> 0) and
              (ffi^.dwFileFlags and VS_FF_PATCHED <> 0) then
                attr := attr + ',patched';
              if (ffi^.dwFileFlagsMask and VS_FF_SPECIALBUILD <> 0) and
              (ffi^.dwFileFlags and VS_FF_SPECIALBUILD <> 0) then
                attr := attr + ',special build';
              if (ffi^.dwFileFlagsMask and VS_FF_PRIVATEBUILD <> 0) and
              (ffi^.dwFileFlags and VS_FF_PRIVATEBUILD <> 0) then
                attr := attr + ',private build';
              if attr <> '' then
                Result := Result + ' ' + Copy(attr, 2, MaxInt);
            end;
          finally
            mem.Free;
          end;
        end;
      finally
        FreeResource(glob);
      end;
    end else
      Result := '<no version info available>';
  except
    Result := '';
  end;
end;

end.
